package com.ruoyi.business.service.impl;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.ruoyi.business.domain.ApplyPlan;
import com.ruoyi.business.domain.Rebot;
import com.ruoyi.business.domain.ResourceLibrary;
import com.ruoyi.business.domain.bo.ApplyStartBO;
import com.ruoyi.business.domain.bo.JobQueryBO;
import com.ruoyi.business.domain.bo.ListApplyBO;
import com.ruoyi.business.domain.bo.ListRebotBO;
import com.ruoyi.business.domain.vo.JobQueryVO;
import com.ruoyi.business.domain.vo.JobStartVO;
import com.ruoyi.business.domain.vo.ListApplyVO;
import com.ruoyi.business.domain.vo.ListRebotVO;
import com.ruoyi.business.service.IApplyPlanService;
import com.ruoyi.business.service.IResourceLibraryService;
import com.ruoyi.business.util.YinDaoHttpUtils;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.business.mapper.ApplyMapper;
import com.ruoyi.business.domain.Apply;
import com.ruoyi.business.service.IApplyService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

/**
 * 应用信息管理Service业务层处理
 *
 * @author LiuChengRan
 * @date 2024-06-13
 */
@Service
public class ApplyServiceImpl extends ServiceImpl<ApplyMapper, Apply> implements IApplyService {

    @Autowired
    private ISysDeptService sysDeptService;

    @Autowired
    private IResourceLibraryService resourceLibraryService;

    @Autowired
    private IApplyPlanService applyPlanService;

    /**
     * 启动应用
     *
     * @param applyStartBO
     */
    @Override
    @Transactional
    public JobStartVO appStart(ApplyStartBO applyStartBO) throws IllegalAccessException {
        JobStartVO jobStartVO = YinDaoHttpUtils.appStart(applyStartBO);
        return jobStartVO;
    }

    /**
     * 查询应用运行结果
     *
     * @param jobQueryBO
     */
    @Override
    public JobQueryVO queryAppStartResult(JobQueryBO jobQueryBO) throws IllegalAccessException {
        JobQueryVO jobQueryVO = YinDaoHttpUtils.queryAppStartResult(jobQueryBO);
        return jobQueryVO;
    }


    /**
     * 同步机器人数据
     *
     * @return
     */
    @Override
    @DataScope(deptAlias = "apply")
    @Transactional
    public void syn(Apply entity) throws IllegalAccessException {
        ListApplyBO listApplyBO = new ListApplyBO();
        listApplyBO.setPage(1);
        // 影刀最大限制
        listApplyBO.setSize(100);
        List<Apply> applyList = baseMapper.selectApplyList(new Apply());
        Map<String, List<Apply>> applyMap = applyList.stream().collect(Collectors.groupingBy(Apply::getAppId));

        List<ListApplyVO> listApplyVOList = YinDaoHttpUtils.listApp(listApplyBO);
        // 获取客户端名称集合
        List<String> appIdList = listApplyVOList.stream().map(ListApplyVO::getAppId).collect(Collectors.toList());
        // 只能移除当前用户部门的
        this.remove(new LambdaUpdateWrapper<Apply>().notIn(Apply::getAppId, appIdList).eq(Apply::getDeptId, SecurityUtils.getDeptId()));

        Map<String, List<SysDept>> deptMap = sysDeptService.selectDeptAllList(new SysDept()).stream().collect(Collectors.groupingBy(SysDept::getDeptName));
        listApplyVOList.forEach(listRebotVO -> {
            // 数据库里是否存在
            List<Apply> applys = applyMap.get(listRebotVO.getAppId());
            String[] deptName = listRebotVO.getAppName().split("-");
            List<SysDept> sysDepts = deptMap.get(deptName[0]);

            if (null == applys) {
                Apply newApply = new Apply();
                BeanUtils.copyBeanProp(newApply, listRebotVO);
                newApply.setAppCreateTime(listRebotVO.getCreateTime());
                newApply.setAppUpdateTime(listRebotVO.getUpdateTime());
                if (null != sysDepts) {
                    newApply.setDeptId(sysDepts.get(0).getDeptId());
                    newApply.setDeptName(sysDepts.get(0).getDeptName());
                }
                save(newApply);
            } else {
                Apply apply = applys.get(0);
                Apply newApply = new Apply();
                BeanUtils.copyBeanProp(newApply, listRebotVO);
                newApply.setAppCreateTime(listRebotVO.getCreateTime());
                newApply.setAppUpdateTime(listRebotVO.getUpdateTime());
                newApply.setId(apply.getId());
                newApply.setSupportParam(listRebotVO.getSupportParam());
                if (null != sysDepts) {
                    newApply.setDeptId(sysDepts.get(0).getDeptId());
                    newApply.setDeptName(sysDepts.get(0).getDeptName());
                }
                // 修改应用名称
                updateById(newApply);
                // 修改参数里应用名称
                ResourceLibrary resourceLibrary = new ResourceLibrary();
                resourceLibrary.setAppName(listRebotVO.getAppName());
                resourceLibraryService.lambdaUpdate().eq(ResourceLibrary::getAppId, apply.getId()).update(resourceLibrary);
                // 修改已执行的计划应用名称
                ApplyPlan applyPlan = new ApplyPlan();
                applyPlan.setAppName(listRebotVO.getAppName());
                applyPlanService.lambdaUpdate().eq(ApplyPlan::getAppId, apply.getId()).update(applyPlan);
            }
        });
    }

    /**
     * 查询应用信息管理列表
     *
     * @param apply 应用信息管理
     * @return 应用信息管理
     */
    @Override
    @DataScope(deptAlias = "apply")
    public List<Apply> list(Apply apply) {
        return baseMapper.selectApplyList(apply);
    }

    /**
     * 查询所有应用信息列表
     *
     * @param apply 应用信息管理
     * @return 应用信息管理
     */
    @Override
    @DataScope(deptAlias = "apply")
    public List<Apply> listAll(Apply apply) {
        return baseMapper.selectApplyAllList(apply);
    }
}
